@@ -13,7 +13,7 @@
 
 package IO::Socket::SSL;
 
-our $VERSION = '2.022';
+our $VERSION = '2.023';
 
 use IO::Socket;
 use Net::SSLeay 1.46;
@@ -1302,6 +1302,11 @@ sub stop_SSL {
 		    # shutdown complete
 		    last;
 		}
+		if ((${*$self}{'_SSL_opened'}||0) <= 0) {
+		    # not really open, thus don't expect shutdown to return
+		    # something meaningful
+		    last;
+		}
 
 		# initiate or complete shutdown
 		local $SIG{PIPE} = 'IGNORE';
